/**
 * @file SensorFSM.c
 * @author your name (you@domain.com)
 * @brief 
 * @version 0.1
 * @date 2022-09-02
 * 
 * @copyright Copyright (c) 2022
 * 
 */

#include "SensorFSM.h"
#include "WIT2FSM.h"
#include "math.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"

sensorFSM_t sensors[6] = {0};

extern bool g_bIsSensorExist[6];

uint32_t u32SensorStateGet(uint32_t addr, uint16_t *state)
{
    return WIT2FSM_SensorStateGet(addr, state);
}

void sensorInit(void)
{
    for (uint32_t i = 0; i < SENSOR_NUMBER_MAX; i++)
    {
		sensors[i].available = g_bIsSensorExist[i];
    }
}

void sensorProcess(void)
{
    uint16_t state = 0;
    // sensor state get
    for (uint32_t i = 0; i < SENSOR_NUMBER_MAX; i++)
    {
        state = 0;
        if (sensors[i].available != true)
        {
            if (sensors[i].errLastUpdateTick != 0)
            {
                uint32_t newTick= xTaskGetTickCount();
                sensors[i].errRestartTick += abs((int)((long long)newTick - (long long)sensors[i].errLastUpdateTick));
                if(sensors[i].errRestartTick > SENSOR_RESTART_TIMEOUT)
                {
                    // sensor resume
                    sensors[i].errLastUpdateTick = 0;
                    sensors[i].errRestartTick = 0;
                    sensors[i].available = true;
                    sensors[i].u32SensorStateErrCnt = 0;
                }
                else
                {
                    sensors[i].errLastUpdateTick = newTick;
                }
                
            }
            
            continue;
        }

        if (u32SensorStateGet(i + 1, &state) != true)
        {
            // sensor state get failed
            sensors[i].state = SEN_STATE_READERROR;
        }
        else
        {
            if (sensors[i].state >= SEN_STATE_MAX)
            {
                // state invaild
                sensors[i].state = SEN_IVLSTATE_ERROR;
            }
            else
            {
                sensors[i].state = state;
            }
            
        }
        
    }
}

sen_state_t sensorStateGet(unsigned int idx)
{
    return sensors[idx % 6].state;
}

bool sensorAvailableGet(uint32_t idx)
{
    return sensors[idx % 6].available;
}

void sensorAvailableSet(uint32_t idx, bool state)
{
    sensors[idx % 6].available = state;
}

uint32_t sensorsAvailableStateGet(void)
{
    uint32_t ret = 0;
    for (size_t i = 0; i < SENSOR_NUMBER_MAX; i++)
    {
        ret |= (sensors[i].available == true) ? (0x02 << i) : 0;
    }
    
    return ret;
}

void sensorsAvailableStateSet(uint32_t available)
{

    if ((available & 126) > 0)
    {
        return;
    }
    for (size_t i = 0; i < SENSOR_NUMBER_MAX; i++)
    {
        sensors[i].available |= ((available & (0x02 << i)) != 0) ? true : false;
    }
    
    return;
}
