/* Includes ------------------------------------------------------------------*/
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "common.h"
#include <stdbool.h>

/* Private macros ------------------------------------------------------------*/
#define COMMON_YEAR_SECONDS 31536000 //平年秒数
#define LEAP_YEAR_SECONDS 31622400	 //闰年秒数
#define ONE_DAY_SECONDS 86400		//一天的秒数
#define ONE_HOUR_SECONDS 3600		//一小时秒数
#define ONE_MINUTE_SECONDS 60		 //一分钟秒数
#define COMMON_YEAR_DAY 365			//平年天数
#define LEAP_YEAR_DAY 366			//闰年天数
#define COMMON_2MONTH_DAY 28	//平年2月天数
#define LEAP_2MONTH_DAY 29		//闰年2月天数
#define SINCE1970 1970

/* Private variables ---------------------------------------------------------*/
static unsigned char g_ucDayOfMonth[] = {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

/* Private functions ---------------------------------------------------------*/
static bool IsLeapYear(unsigned short usYear);

/**
 * @brief  Delay us
 * @param  dTime_us: t us
 * @retval None
 */
void Delay_us(double dTime_us)
{
	volatile unsigned long tmpCounter = dTime_us * 10;

	while (tmpCounter)
	{
		tmpCounter--;
	}
}

/**
 * @brief  Delay ms
 * @param  ulTime_ms: t ms
 * @retval None
 */
void Delay_ms(double dTime_ms)
{
	Delay_us(dTime_ms * 1000);
}

/**
 * @brief  Delay s
 * @param  ulTime_s: t s
 * @retval None
 */
void Delay_s(double dTime_s)
{
	Delay_ms(dTime_s * 1000);
}

/**
 * @brief  Split data
 * @param
 * @retval
 */
void Split(char *pcSplitedData[], unsigned char ucSplitedArraySize, char *pcSource, char *pcSymbol)
{
	char *pcPointer = pcSource;
	char *pcTemp = NULL;
	unsigned char ucIndex = 0;

	if (pcSplitedData == NULL || pcSource == NULL || pcSymbol == NULL)
		return;

	while (true)
	{
		if ((pcTemp = strstr(pcPointer, pcSymbol)) == NULL)
		{
			pcSplitedData[ucIndex++] = pcPointer;
			break;
		}
		else
		{
			memset(pcTemp, 0, strlen(pcSymbol));
			pcSplitedData[ucIndex++] = pcPointer;
			pcPointer = pcTemp + strlen(pcSymbol);
		}

		if (ucIndex >= ucSplitedArraySize)
			break;
	}
}

/**
 * @brief  Cut string
 * @param
 * @retval None
 */
void CutString(char *pcDestination, unsigned long ulDestinationSize, char *pcSource, char *pcStart, char *pcEnd)
{
	char *pcS, *pcE;
	if (pcSource == NULL || pcStart == NULL || pcEnd == NULL)
		return;

	pcS = strstr(pcSource, pcStart);
	pcE = strstr(pcS, pcEnd);

	if (pcS && pcE)
	{
		if (pcE - pcS < ulDestinationSize)
		{
			unsigned long ulSize = pcE - pcS;

			if (pcDestination != NULL)
			{
				memcpy(pcDestination, pcS, ulSize);
				pcDestination[ulSize] = '\0';
			}
		}
	}
}
/**
 * @brief  Time to time stamp since 1970
 * @param
 * @retval None
 */
unsigned long TimeToTimeStamp(Calendar *pxCalendar)
{
	unsigned char ucCurrentMonth;
	unsigned short usCurrentYear;
	unsigned long ulTimeStamp = 0;

	if (pxCalendar == NULL)
		return 0;

	//从1970年到今年的秒数
	for (usCurrentYear = SINCE1970; usCurrentYear < pxCalendar->usYear; usCurrentYear++)
	{
		if (IsLeapYear(usCurrentYear))
		{
			ulTimeStamp += LEAP_YEAR_SECONDS;
		}
		else
		{
			ulTimeStamp += COMMON_YEAR_SECONDS;
		}
	}

	//到当月的秒数
	for (ucCurrentMonth = 1; ucCurrentMonth < pxCalendar->ucMonth; ucCurrentMonth++)
	{
		ulTimeStamp += g_ucDayOfMonth[ucCurrentMonth] * ONE_DAY_SECONDS;
		if (ucCurrentMonth == 2 && IsLeapYear(usCurrentYear))
		{
			ulTimeStamp += ONE_DAY_SECONDS;
		}
	}

	//到当日的秒数
	if (pxCalendar->ucDay > 0)
		ulTimeStamp += (pxCalendar->ucDay - 1) * ONE_DAY_SECONDS;

	//到当时的秒数
	ulTimeStamp += pxCalendar->ucHour * ONE_HOUR_SECONDS;
	ulTimeStamp += pxCalendar->ucMinute * ONE_MINUTE_SECONDS;
	ulTimeStamp += pxCalendar->ucSecond;

	return ulTimeStamp;
}

/**
 * @brief  Time stamp to time
 * @param
 * @retval None
 */
void TimeStampToTime(unsigned long ulTimeStamp, Calendar *pxCalendar)
{
	unsigned long ulTotalDay = ulTimeStamp / ONE_DAY_SECONDS;
	unsigned long ulDaySeconds = ulTimeStamp % ONE_DAY_SECONDS;
	unsigned short usCurrentYear = 1970;
	unsigned char ucCurrentMonth = 1;

	if (pxCalendar == NULL)
		return;

	//根据天数推算出现在是哪一年
	while (ulTotalDay >= COMMON_YEAR_DAY)
	{
		if (IsLeapYear(usCurrentYear))
		{
			if (ulTotalDay >= LEAP_YEAR_DAY)
			{
				ulTotalDay -= LEAP_YEAR_DAY;
			}
			else
			{
				break;
			}
		}
		else
		{
			ulTotalDay -= COMMON_YEAR_DAY;
		}
		usCurrentYear++;
	}

	//根据剩余天数推算出现在是哪一月
	while (ulTotalDay >= COMMON_2MONTH_DAY)
	{
		if (IsLeapYear(usCurrentYear) && ucCurrentMonth == 2)
		{
			if (ulTotalDay >= LEAP_2MONTH_DAY)
			{
				ulTotalDay -= LEAP_2MONTH_DAY;
			}
			else
			{
				break;
			}
		}
		else
		{
			if (ulTotalDay >= g_ucDayOfMonth[ucCurrentMonth])
			{
				ulTotalDay -= g_ucDayOfMonth[ucCurrentMonth];
			}
			else
			{
				break;
			}
		}
		ucCurrentMonth++;
	}

	pxCalendar->usYear = usCurrentYear;
	pxCalendar->ucMonth = ucCurrentMonth;
	pxCalendar->ucDay = ulTotalDay + 1;
	pxCalendar->ucHour = ulDaySeconds / ONE_HOUR_SECONDS;
	pxCalendar->ucMinute = (ulDaySeconds % ONE_HOUR_SECONDS) / ONE_MINUTE_SECONDS;
	pxCalendar->ucSecond = (ulDaySeconds % ONE_HOUR_SECONDS) % ONE_MINUTE_SECONDS;
}

/**
 * @brief  Byte to BCD
 * @param
 * @retval None
 */
unsigned char ByteToBCD(unsigned char ucByte)
{
	unsigned long ulBCDHigh = 0;

	while (ucByte >= 10)
	{
		ulBCDHigh++;
		ucByte -= 10;
	}
	return ((unsigned char)(ulBCDHigh << 4) | ucByte);
}

/**
 * @brief  BCD to byte
 * @param
 * @retval None
 */
unsigned char BCDToByte(unsigned char ucBCD)
{
	unsigned long ulTmp = 0;
	ulTmp = ((unsigned char)(ucBCD & (unsigned char)0xF0) >> (unsigned char)0x4) * 10;
	return (ulTmp + (ucBCD & (unsigned char)0x0F));
}

/**
 * @brief  is leap year?
 * @param
 * @retval true / false
 */
static bool IsLeapYear(unsigned short usYear)
{
	if ((usYear % 4 == 0 && usYear % 100 != 0) || (usYear % 400 == 0))
	{
		return true; //是闰年
	}
	else
	{
		return false; //不是闰年
	}
}

/**
 * @brief  get flag
 * @param
 * @retval true / false
 */
bool GetFlagStatus(unsigned long *pulFlag, unsigned char ucBitPosition)
{
	if (pulFlag != NULL)
		return (bool)(*pulFlag & (1 << ucBitPosition)) > 0 ? true : false;

	return false;
}

/**
 * @brief  set flag
 * @param
 * @retval None
 */
void SetFlagStatus(unsigned long *pulFlag, unsigned char ucBitPosition, bool bStatus)
{
	if (pulFlag != NULL)
		*pulFlag |= (bStatus << ucBitPosition);
}

/**
 * @brief  find string in memory
 * @param
 * @retval None
 */
char *memstr(void *pvSrc, char *pcStr, unsigned long ulLength)
{
	char *pcSource = NULL;
	char *pcResult = NULL;
	unsigned long ulHandlerLength = 0;
	unsigned long ulCurLength = 0;

	if (pvSrc == NULL || pcStr == NULL)
		return NULL;

	pcSource = (char *)pvSrc;
	while (ulHandlerLength < ulLength)
	{
		if ((pcResult = strstr(pcSource, pcStr)) != NULL)
			break;

		ulCurLength = strlen(pcSource);
		ulHandlerLength += ulCurLength == 0 ? 1 : ulCurLength;
		pcSource += ulCurLength == 0 ? 1 : ulCurLength;
	}

	return pcResult;
}

/* @brief  replace specific string
 * @param
 * @retval None
 */
bool Replace(char *pcOrigin, char *pcSearch, char *pcReplace, char *pcDest, unsigned long ulMaxLength)
{
	char *pcHead = pcOrigin;
	char *pcTail = pcOrigin;
	unsigned long ulCurrentLen = 0;

	if (pcOrigin == NULL || pcSearch == NULL || pcReplace == NULL || pcDest == NULL)
		return false;
	memset(pcDest, 0, ulMaxLength);
	while ((pcTail = strstr(pcHead, pcSearch)) != NULL)
	{
		ulCurrentLen += (pcTail - pcHead) + strlen(pcReplace);

		if (ulCurrentLen >= ulMaxLength)
			return false;
		strncat(pcDest, pcHead, pcTail - pcHead);
		strcat(pcDest, pcReplace);
		pcHead = pcTail + strlen(pcSearch);
	}
	ulCurrentLen += strlen(pcHead);
	if (ulCurrentLen >= ulMaxLength)
		return false;
	strcat(pcDest, pcHead);

	return true;
}

/** @brief  Fixed decimal size
 * @param
 * @retval double type value
 */
double Round(double dValue, unsigned char ucPrecision)
{
	char cFormat[8] = "";
	char cDoubleValue[20] = "";

	sprintf(cFormat, "%%.%dlf", ucPrecision);
	sprintf(cDoubleValue, cFormat, dValue);
	dValue = atof(cDoubleValue);
	return dValue;
}
