#ifndef __IF_BLE_CONFIG__H__
#define __IF_BLE_CONFIG__H__

#include "wcp.h"
#include "acc.h"

#define RETARY_CNT                                1
#define WIT_SENSOR_GROUP_MAX_NUM                  8
#define SENSOR_MAX_FREQ_X                         10
#define SENSOR_BEARING_LOGITIC_MAX_SIZE           12
#define SENSOR_GEAR_LOGITIC_MAX_SIZE              6
#define SENSOR_WAVE_MAX_SIZE                     (16 * 1024)
//轴承最大数量
#define SENSOR_BEARING_MAX_COUNT                  2
//齿轮最大数量
#define SENSOR_GEAR_MAX_COUNT                     3

#define SENSOR_PARTS_MAX_NUM                      10
// define
#define BEARING_CONFIG_VAR(type, name, n) \
	type name[n];                         \
	type name##Reserved[SENSOR_BEARING_LOGITIC_MAX_SIZE - n]
#define GEAR_CONFIG_VAR(type, name, n) \
	type name[n];                      \
	type name##Reserved[SENSOR_GEAR_LOGITIC_MAX_SIZE - n]

typedef enum
{
	Status_Idle        = 0,	   //空闲
	Status_Collecting  = 1,    //采集中
	Status_Completed   = 2,    //采集完成
	Status_Error       = 3,	   //采集失败
} Sensor_Status;

/* Type defines --------------------------------------------------------------*/
typedef enum
{
	Mode_Passive = 0,   //被动模式
	Mode_Auto    = 1,	//自动模式
} Sensor_Mode;

typedef struct
{
	float fPassFrequency;
	bool  bEnable;		           //是否启用
	unsigned short usAlias;        //别名
} PwrFreq;

typedef struct
{
	/* Base Config */
	unsigned long  ulAddr;
	unsigned long  ulSpeed;
	unsigned short usPolePairs;
	unsigned short usSamplingCount;
	unsigned short usAutoModeSamplingPeriod;
	short sXOffset;
	short sYOffset;
	short sZOffset;
	unsigned long ulSerialNumber;
	unsigned long ulConfigNumber;
	/* Component Config */
	bool bSetSpeed;
	// bearing enabled status (兼容12轴承 6齿轮版本)
	BEARING_CONFIG_VAR(bool, bBearingEnabled, SENSOR_BEARING_MAX_COUNT);
	// gear enabled status
	GEAR_CONFIG_VAR(bool, bGearEnabled, SENSOR_GEAR_MAX_COUNT);
	// bearing config parm
	BEARING_CONFIG_VAR(Bearing, xBearing, SENSOR_BEARING_MAX_COUNT);
	// gear config parm
	GEAR_CONFIG_VAR(Gear, xGear, SENSOR_GEAR_MAX_COUNT);
	unsigned long ulComponentConfigNumber;
	/* Parm1 Config */
	bool bAutoDeleteGravity;
	unsigned long ulFrequencyRange;
	unsigned long ulParmConfigNumber;
	/* Parm2 Config */
	Sensor_Mode eMode;
	bool bCalculateFeature;
	unsigned long ulParm2ConfigNumber;
	/* Active Config */
	unsigned short usActiveThreshold;
	unsigned short usActiveRatio;
	unsigned long ulActiveParmConfigNumber;
	/* Extend Config */
	// bearing extend enabled status (2轴承 3齿轮)
	bool bBearingExtendEnable[SENSOR_BEARING_MAX_COUNT];
	// gear extend enabled status
	bool bGearExtendEnable[SENSOR_GEAR_MAX_COUNT];
	Base_ExtendConfig xBaseExtendConfig;
	// bearing extend config
	Bearing_ExtendConfig xBearingExtendConfig[SENSOR_BEARING_MAX_COUNT];
	// gear extend config
	Gear_ExtendConfig xGearExtendConfig[SENSOR_GEAR_MAX_COUNT];
	unsigned long ulExtendParmConfigNumber;
	bool bFanEnable;
	bool bBeltEnable;
	Fan xFan;
	Belt xBelt;
	unsigned long ulFanAndBeltConfigNumber;
	unsigned short usFanAlias;
	unsigned short usBeltAlias;
	unsigned long ulFanAndBeltAliasConfigNumber;

	bool bScrewEnable;
	Screw xScrew;
	unsigned short usScrewAlias;
	unsigned long ulScrewConfigNumber;
	unsigned long ulScrewAliasConfigNumber;

	bool bMaleScrewEnable;
	MaleScrew xMaleScrew;
	unsigned short usMaleScrewAlias;
	unsigned long ulMaleScrewConfigNumber;
	unsigned long ulMaleScrewAliasConfigNumber;

	bool bFemaleScrewEnable;
	FemaleScrew xFemaleScrew;
	unsigned short usFemaleScrewAlias;
	unsigned long ulFemaleScrewConfigNumber;
	unsigned long ulFemaleScrewAliasConfigNumber;

	bool bShaftEnable;
	Shaft xShaft;
	unsigned short usShaftAlias;
	unsigned long ulShaftConfigNumber;
	unsigned long ulShaftAliasConfigNumber;

	unsigned int odr;
	unsigned int amlVibTh;
	unsigned int amlVibInterval;
	unsigned int amlVibKeepCnt;
	unsigned int highTempProtect;
	unsigned long v2_20ConfigNumber;

	unsigned int vRMSMode;
	bool bSleeveEnable;
	Sleeve xSleeve;
	unsigned short usSleeveAlias;
	unsigned long ulSleeveConfigNumber;
	unsigned long ulSleeveAliasConfigNumber;
	PwrFreq  xPwrFreq;
	unsigned long ulPowerFrequencyConfigNumber;
	unsigned short usxVar;
} Sensor_Config;

void            if_sensor_config_init(void);
bool            if_sensor_config(unsigned int index, unsigned char point, unsigned long *ulConfigEnable);
Sensor_Config   *getSensorConfig(unsigned char index);
void            if_clear_config_flag(unsigned char index);
void            if_falsh_write_config(void);
void            if_falsh_read_config(void);
bool            if_synchronize_time(unsigned int index, unsigned long ulSyncTime, bool bCheck);
bool            if_set_collect_time(unsigned int index, unsigned long ulConnectTime, bool bCheck);
bool            if_set_wakeup_time(unsigned int index, unsigned long ulWakeUpTime, unsigned long ulWakeUpInterval, bool bCheck);
void            if_check_change_config(void);
void            if_printf(void);
#endif
