/* Includes ------------------------------------------------------------------*/
#include <string.h>
#include "mqtt_update.h"
#include "esp_ota_ops.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/semphr.h"
#include "freertos/timers.h"
#include "esp_log.h"
#include "flash.h"
#include "wit.h"
#include "ble_client.h"
#include "if_ble_config.h"

#define SENSOR_UPDATE_MIN_SIZE             6
#define SENSOR_UPDATE_TIMEOUT              60000
#define MQTT_SENSOR_UPDATE_PER_SIZE        4096

static bool g_bSensorUpdate = false;
static TimerHandle_t g_xSensorUpdateTimer = NULL;
static Update_ResponseStatus g_eUpdateResponse = Response_Null;
static unsigned short g_ucTotalUpdateSize = 0;
static unsigned short g_ucCurrentPackageID = 0;
static unsigned int   offset = 0;
static unsigned int   gBCurrent = 0;

static void Sensor_TimeoutHandler(TimerHandle_t xTimer);

void Sensor_UpdateHandler(char *pcGWUpdateString, unsigned long ulLen)
{
	unsigned long ulReceivedCheckSum = 0;
	unsigned long ulCalculatedCheckSum = 0;
	char *pcDataPointer = NULL;
	unsigned long ulDataLen = 0;
	int i;

	if (ulLen < SENSOR_UPDATE_MIN_SIZE)
		return;
    g_eUpdateResponse = Response_Null;
	// get current package id
	g_ucCurrentPackageID = ((short)pcGWUpdateString[1]) |((short)pcGWUpdateString[0] << 8);
	// get package check sum
	ulReceivedCheckSum = (unsigned long)pcGWUpdateString[ulLen - 4] << 24 | (unsigned long)pcGWUpdateString[ulLen - 3] << 16 |
						 (unsigned long)pcGWUpdateString[ulLen - 2] << 8 | (unsigned long)pcGWUpdateString[ulLen - 1];
    // esp_log_buffer_hex("updata", pcGWUpdateString, ulLen);	
	// calculate check sum
	for (i = 0; i < ulLen - 4; i++)
	{
		ulCalculatedCheckSum += pcGWUpdateString[i];
	}
	// compare
	if (ulCalculatedCheckSum == ulReceivedCheckSum)
	{
		pcDataPointer = &pcGWUpdateString[2];
		ulDataLen = ulLen - SENSOR_UPDATE_MIN_SIZE;
		if (g_ucCurrentPackageID == 0)
		{
			gBCurrent = 0;
			offset = 0;
			// check flag
			if (ulDataLen - 2 == strlen("Witium_Update") && memcmp(pcDataPointer, "Witium_Update", ulDataLen - 2) == 0)
			{
				// Timer Init
				if (g_xSensorUpdateTimer == NULL)
					g_xSensorUpdateTimer = xTimerCreate("SensorTimer", SENSOR_UPDATE_TIMEOUT, pdFALSE, NULL, Sensor_TimeoutHandler);

				// Timer start
				if (g_xSensorUpdateTimer != NULL)
					xTimerStart(g_xSensorUpdateTimer, 1000);
				
				// get total package
				g_ucTotalUpdateSize = ((short)pcGWUpdateString[ulLen - 5]) |((short)pcGWUpdateString[ulLen - 6] << 8);
				erase_bin();
				ESP_LOGD("ble_ud","%d", (int)g_ucTotalUpdateSize);
				g_bSensorUpdate = true;
				g_eUpdateResponse = Response_Success;
			}
		}
		else if (g_bSensorUpdate == true)
		{
			if (g_ucCurrentPackageID < g_ucTotalUpdateSize)
			{
				if (ulDataLen == MQTT_SENSOR_UPDATE_PER_SIZE)
				{
					// restart timer
					if (g_xSensorUpdateTimer != NULL)
						xTimerStart(g_xSensorUpdateTimer, 1000);
					// write data
					if(gBCurrent == g_ucCurrentPackageID) {
						esp_err_t ret;
						ret = write_ble_bin((char *)pcDataPointer, offset, ulDataLen);
						// ESP_LOGD("ble_update","offset = %d, recv_size = %d, id = %d", (int)offset, (int)ulDataLen, g_ucCurrentPackageID);
						
						if(ret != ESP_OK)
						{
							gBCurrent = 0;
							g_bSensorUpdate = false;
							offset = 0;
							g_eUpdateResponse = Response_Error;
						}
						else
						{
							offset = offset + ulDataLen;
							g_eUpdateResponse = Response_Success;
						}
					} else {
						gBCurrent = 0;
						g_bSensorUpdate = false;
						offset = 0;
						g_eUpdateResponse = Response_Error;
					}
				} else {
					gBCurrent = 0;
					g_bSensorUpdate = false;
					offset = 0;
					g_eUpdateResponse = Response_Error;	
				}
			}
			else if (g_ucCurrentPackageID == g_ucTotalUpdateSize)
			{
				// stop timer
				if (g_xSensorUpdateTimer != NULL)
					xTimerStop(g_xSensorUpdateTimer, 1000);
				// write data
				
				if(gBCurrent == g_ucCurrentPackageID) {
					gBCurrent = 0;
					write_ble_bin((char *)pcDataPointer, offset, ulDataLen);
				    // esp_log_buffer_hex("updata", pcDataPointer, ulDataLen);	
					// ESP_LOGD("ble_update","offset = %d, recv_size = %d, id = %d", (int)offset, (int)ulDataLen, g_ucCurrentPackageID);
				    offset = offset + ulDataLen;
					g_eUpdateResponse = Response_Reset;
				} else{
					gBCurrent = 0;
					g_bSensorUpdate = false;
					offset = 0;
					g_eUpdateResponse = Response_Error;
				}
			}
		}
	}

	if (g_eUpdateResponse == Response_Null)
		g_eUpdateResponse = Response_Error;
	gBCurrent++;		
}

bool Sensor_GetUpdateStatus(void)
{
	return g_bSensorUpdate;
}

static void Sensor_TimeoutHandler(TimerHandle_t xTimer)
{
	g_bSensorUpdate = false;
}

Update_ResponseStatus BLE_Update_Poll(unsigned char *pucCurrentPackageID)
{
	Update_ResponseStatus eStatus = Response_Null;
	if (g_eUpdateResponse != Response_Null)
	{
		switch (g_eUpdateResponse)
		{
		case Response_Reset:
			// end the OTA and start validation
			gBCurrent = 0;
			g_bSensorUpdate = false;
            setBleBinSize(offset);
			// setUpdateFlag(1);				
			offset = 0;
			break;
		default:
			break;
		}

		// return status
		eStatus = g_eUpdateResponse;
		// get package id
		*pucCurrentPackageID = g_ucCurrentPackageID;
		g_eUpdateResponse = Response_Null;
	}

	return eStatus;
}