/* Includes ------------------------------------------------------------------*/
#include "quectel_3gpp.h"

#if QUECTEL > 0
#if CELLUAR_TCPIP_ENABLE > 0 || CELLUAR_SMS_ENABLE > 0
/**
  * @brief  Quectel whether has card?
  * @param  None
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_GetCardStatus()
{
	Celluar_ErrorStatus eResult;
	char cCommand[12], cResponse[48];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CPIN?\r");
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "+CPIN:", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_500MS)) == Celluar_OK)
	{
		// printf("AT+CPIN : %s",cResponse);
		if (!strstr(cResponse, "READY"))
		{
			eResult = Celluar_Error;
		}
	}
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Quectel get signal
  * @param  ucRssi: 0-31 / 99(not known or not detetable)
  * @param  ucBer: 0-7 / 99(not known or not detetable)
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_GetSignal(unsigned char *pucRssi, unsigned char *pucBer)
{
	Celluar_ErrorStatus eResult;
	char cCommand[12], cResponse[48];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CSQ\r");
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "+CSQ:", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_500MS)) == Celluar_OK)
	{
		char *pcTemp = NULL;
		char *pcSplitedData[2] = {NULL, NULL};
		pcTemp = strstr(cResponse, "+CSQ:") + strlen("+CSQ:");
		Split(pcSplitedData, 2, pcTemp, ",");
		if (pcSplitedData[0] && pcSplitedData[1])
		{
			if (pucRssi != NULL)
				*pucRssi 	= atol(pcSplitedData[0]);
			if (pucBer != NULL)
				*pucBer 	= atol(pcSplitedData[1]);
		}
		else
		{
			eResult = Celluar_Error;
		}
	}
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Quectel get registration status
  * @param  None
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_GetRegistrationStatus()
{
	Celluar_ErrorStatus eResult;
	char cCommand[12], cResponse[64];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CREG?\r");
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "+CREG:", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_500MS)) == Celluar_OK)
	{
		char *pcTemp = NULL;
		char *pcSplitedData[2] = {NULL, NULL};
		long lStatus;
		pcTemp = strstr(cResponse, "+CREG:") + strlen("+CREG:");
		Split(pcSplitedData, 2, pcTemp, ",");
		if (pcSplitedData[0] && pcSplitedData[1])
		{
			lStatus = atol(pcSplitedData[1]);
			if (lStatus != 1 && lStatus != 5)
			{
				eResult = Celluar_Error;
			}
		}
		else
		{
			eResult = Celluar_Error;
		}
	}
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Quectel get ccid
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_GetCCID(char *pcCCID, unsigned long ulBufferLen)
{
	Celluar_ErrorStatus eResult;
	char cCommand[12];
	char cCCID[60] = {0};
	
	Celluar_Take();
	sprintf(cCommand, "AT+QCCID\r");
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "+QCCID:", cCCID, sizeof(cCCID), NULL, CELLUAR_WAITTIME_500MS)) == Celluar_OK)
	{
		char *pcStart, *pcEnd;
		
		pcStart = strstr(cCCID, "+QCCID: ") + strlen( "+QCCID: ");
		if ((pcEnd = strstr(pcStart, "\r\n")) != NULL)
		{
			*pcEnd = '\0';
			
			if (ulBufferLen > strlen(pcStart))
			{
				strcpy(pcCCID, pcStart);
			}
			else
			{
				eResult = Celluar_Error;
			}
		}
		else
		{
			eResult = Celluar_Error;
		}
	}
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Quectel set registration mode
  * @param  ulMode: 0 1 2
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_SetNetworkRegistrationMode(unsigned char ucMode)
{
	Celluar_ErrorStatus eResult;
	char cCommand[13];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CREG=%d\r", ucMode);
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, 0, NULL, CELLUAR_WAITTIME_500MS);
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Quectel set COPS mode
  * @param  ulMode: 0 1 2 3 4
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_SetCOPSMode(unsigned char ucMode, unsigned char ucFormat)
{
	Celluar_ErrorStatus eResult;
	char cCommand[17];
	
	Celluar_Take();
	sprintf(cCommand, "AT+COPS=%d,%d\r", ucMode, ucFormat);
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, 0, NULL, CELLUAR_WAITTIME_3M);
	Celluar_Give();
	return eResult;
}
#endif
#endif
