/* Includes ------------------------------------------------------------------*/
#include "quectel_http.h"
#include "quectel_file.h"

#if QUECTEL > 0
#if CELLUAR_HTTP_ENABLE > 0
/**
  * @brief  Quectel http set url
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus	Quectel_SetHTTPURL(char *pcURL)
{
	Celluar_ErrorStatus eResult;
	char cCommand[200];
	
	if (strlen(pcURL) >= 160)
		return Celluar_Error;
		
	
	Celluar_Take();
	sprintf(cCommand, "AT+QHTTPURL=%d\r", strlen(pcURL));
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "CONNECT", NULL, NULL, NULL, CELLUAR_WAITTIME_500MS);
	if (eResult == Celluar_OK)
	{
		eResult = Celluar_SendCommandAndWaitForResponse(pcURL, strlen(pcURL), "OK", NULL, NULL, NULL, CELLUAR_WAITTIME_500MS);
	}
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Quectel http get / timeout:default 60s
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_HTTPGetMethod(unsigned long *pulResponseLen, unsigned long *pulErr)
{
	Celluar_ErrorStatus eResult;
	char cCommand[16], cResponse[48];
	
	Celluar_Take();
	sprintf(cCommand, "AT+QHTTPGET\r");
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "+QHTTPGET:", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_2M)) == Celluar_OK)
	{
		char *pcSplitedData[3] = {NULL, NULL, NULL};
		
		Split(pcSplitedData, 3, cResponse, ",");
		if (pcSplitedData[1] != NULL && pcSplitedData[2] != NULL)
		{
			unsigned short usStatus;
			
			usStatus = atol(pcSplitedData[1]);
			if (pulErr != NULL)
				*pulErr = usStatus;
			if (usStatus != 200)
			{
				eResult = Celluar_Error;
			}
			else
			{
				if (pulResponseLen != NULL)
				{
					*pulResponseLen = atol(pcSplitedData[2]);
				}
			}
		}
	}
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Quectel http read server response / timeout:default 60s
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_ReadHTTPServerResponseFromRAM(unsigned long ulStartAddress, unsigned long ulSize, unsigned char *pucResponse, unsigned long ulBufferLen, unsigned long *pulRespLen)
{
	Celluar_ErrorStatus eResult = Celluar_Error;
	unsigned long ulFileHandle = 3000;
	
	if (Quectel_FileOpen("RAM:response.txt", 2, &ulFileHandle) == Celluar_OK)
	{
		if (Quectel_FileSeek(ulFileHandle, ulStartAddress) == Celluar_OK)
		{
			if (Quectel_FileRead(ulFileHandle, ulSize, pucResponse, ulBufferLen, pulRespLen) == Celluar_OK)
			{
				eResult = Celluar_OK;
			}
		}
	}
	//Close file
	Quectel_FileClose(ulFileHandle);
	
	return eResult;
}

/**
  * @brief  Quectel http store server response / timeout:default 60s
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_StoreHTTPServerResponseToRAM()
{
	Celluar_ErrorStatus eResult;
	char cCommand[48], cResponse[48];
	
	Celluar_Take();
	sprintf(cCommand, "AT+QHTTPREADFILE=\"RAM:response.txt\"\r");
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "+QHTTPREADFILE:", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_2M)) == Celluar_OK)
	{
		if (strstr(cResponse, "+QHTTPREADFILE: 0") == NULL)
		{
			eResult = Celluar_Error;
		}
	}
	Celluar_Give();
	return eResult;
}


Celluar_ErrorStatus Quectel_TerminateHTTPService()
{
	//No response in EC20
//	Celluar_ErrorStatus eResult;
//	char cCommand[48];
//	
//	Celluar_Take();
//	sprintf(cCommand, "AT+QHTTPSTOP\r");
//	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, NULL, NULL, CELLUAR_WAITTIME_10S);
//	Celluar_Give();
//	return eResult;
	
	return Celluar_OK;
}
#endif
#endif

