#ifndef ACC_H
#define ACC_H
/* Includes ------------------------------------------------------------------*/
#include "common.h"

/* Exported macros -----------------------------------------------------------*/
#define ACC_MAX_BUFFER_COUNT         100
#define ACC_MAX_WAVE_SUPPORT_COUNT   2730
//轴承最大数量
#define ACC_MAX_BEARING_COUNT 2
//齿轮最大数量
#define ACC_MAX_GEAR_COUNT 3
// Others
#define ACC_AXIS_COUNT 3
#define ACC_BASIC_FEATURE_READ_COUNT 9
#define ACC_FREQUENCY_FEATURE_COUNT 3

#if defined(__CC_ARM)
#pragma anon_unions
#endif

/* Exported types ------------------------------------------------------------*/
typedef enum
{
	ACC_STATE_IDLE,
	ACC_STATE_COLLECTING,
	ACC_STATE_COMPLETED,
	ACC_STATE_ERROR,
	ACC_STATE_RECALCULATE,

	ACC_STATE_MAX
} AccState;

typedef struct
{
	short sWaveArray[ACC_MAX_BUFFER_COUNT]; //波形临时缓冲
	unsigned long ulWaveCount[24];				//点数
} Wave;

typedef struct
{
	Wave xWave;
} Sensor_Wave;

//特征值定义
typedef struct
{
	float fValue;	//参数值
	char cName[32]; //参数名
	union
	{
		unsigned char ucAlarmAddressIndex; //预警值索引地址
		unsigned char ucErrorOccuredCount; //错误次数
	};
	union
	{
		bool bIsHighLimitAlarm; //超过上限预警
		bool bIsErrorOccured;	//故障发生
	};
	union
	{
		bool bIsLowLimitAlarm; //超过下限预警
		bool bIsErrorCleared;  //故障清除
	};
	bool bIsUsed; //是否使用参数
} Parameter;

typedef struct
{
	Parameter xFreqBO; //外圈频点
	Parameter xFreqBI; //内圈频点
	Parameter xFreqBB; //滚珠频点
	Parameter xFreqBF; //保持架频点
} Bearing_Parameter;

typedef struct
{
	Parameter xFreq1xGMF; // 1x啮合频率
	Parameter xFreq2xGMF; // 2x啮合频率
	Parameter xFreq3xGMF; // 3x啮合频率
} Gear_Parameter;

typedef struct
{
	Parameter xSPM1x;
	Parameter xSPM2x;
	Parameter xSPM3x;
} Fan_Parameter;

typedef struct
{
	Parameter xBelt1x;
	Parameter xSelfVirb;
} Belt_Parameter;

typedef struct
{
	Parameter xScrew1x;
	Parameter xScrew2x;
	Parameter xScrew3x;
} Screw_Parameter;

typedef struct
{
	Parameter xMaleScrew1x;
	Parameter xMaleScrew2x;
	Parameter xMaleScrew3x;
} MaleScrew_Parameter;

typedef struct
{
	Parameter xFemaleScrew1x;
	Parameter xFemaleScrew2x;
	Parameter xFemaleScrew3x;
} FemaleScrew_Parameter;

typedef struct
{
	Parameter xShaft1x;
	Parameter xShaft2x;
	Parameter xShaft3x;
} Shaft_Parameter;

typedef struct
{
	Parameter xShaftA05xFn;
	Parameter xShaftA1xFn;
	Parameter xShaftA2xFn;
	Parameter xShaftA3xFn;
	Parameter xShaftA4xFn;
	Parameter xShaftA5xFn;
	Parameter xShaftA6xFn;
	Parameter xShaftA7xFn;
	Parameter xShaftA8xFn;
	Parameter xShaftA9xFn;
	Parameter xShaftA10xFn;
	Parameter xmaxFreqA05xFn;
} Sleeve_Parameter;

typedef struct
{
	Parameter xNaAmFreq;
	Parameter xEnergy;
} PowerFrequency_Parameter;

#if CONFIG_RS485_RTU > 0
typedef struct
{
	Parameter xMean;									  //均值
	Parameter xRMS;										  //均方根
	Parameter xHighFreqRMS;								  //高频均方根(10-2KHz)
	Parameter xStd;										  //标准差
	Parameter xVar;										  //方差
	Parameter xPeak;									  //峰值
	Parameter xCrestFactor;								  //波峰因素
	Parameter xKurtosis;								  //峭度
	Parameter xVRMS;									  //速度均方根
	Parameter xFreq1x;									  // 1倍频
	Parameter xFreq2x;									  // 2倍频
	Parameter xFreq3x;									  // 3倍频
	Bearing_Parameter xBearingFea[ACC_MAX_BEARING_COUNT]; //轴承特征
	Gear_Parameter xGearFea[ACC_MAX_GEAR_COUNT];		  //齿轮特征
	Fan_Parameter xFan;									  //风机特征
	Belt_Parameter xBelt;								  //皮带特征
	Screw_Parameter xScrew;
	MaleScrew_Parameter xMaleScrew;
	FemaleScrew_Parameter xFemaleScrew;
	Shaft_Parameter xShaft;
	Sleeve_Parameter xSleeve;
	PowerFrequency_Parameter xPowerFrequency;
} Feature;
#endif

#if CONFIG_BLUETOOTH > 0
typedef struct
{
	Parameter xMean;									  //均值
	Parameter xRMS;										  //均方根
	Parameter xHighFreqRMS;								  //高频均方根(10-2KHz)
	Parameter xStd;										  //标准差
	Parameter xVar;										  //方差
	Parameter xPeak;									  //峰值
	Parameter xCrestFactor;								  //波峰因素
	Parameter xKurtosis;								  //峭度
	Parameter xVRMS;									  //速度均方根
	Parameter xFreq05x;									  // 0.5倍频
	Parameter xFreq10x;									  // 1.0倍频
	Parameter xFreq15x;									  // 1.5倍频
	Parameter xFreq20x;									  // 2.0倍频
	Parameter xFreq25x;									  // 2.5倍频
	Parameter xFreq30x;									  // 3.0倍频
	Parameter xFreq35x;									  // 3.5倍频
	Parameter xFreq40x;									  // 4.0倍频
	Parameter xFreq45x;									  // 4.5倍频
	Parameter xFreq50x;									  // 5.0倍频
	Parameter xFreq55x;									  // 5.5倍频
	Parameter xFreq60x;									  // 6.0倍频
	Parameter xFreq65x;									  // 6.5倍频
	Parameter xFreq70x;									  // 7.0倍频
	Parameter xFreq75x;									  // 7.5倍频
	Parameter xFreq80x;									  // 8.0倍频
	Parameter xFreq85x;									  // 8.5倍频
	Parameter xFreq90x;									  // 9.0倍频
	Parameter xFreq95x;									  // 9.5倍频
	Parameter xFreq100x;								  // 10.0倍频
	Bearing_Parameter xBearingFea[ACC_MAX_BEARING_COUNT]; //轴承特征
	Gear_Parameter xGearFea[ACC_MAX_GEAR_COUNT];		  //齿轮特征
	Fan_Parameter xFan;									  //风机特征
	Belt_Parameter xBelt;								  //皮带特征
	Screw_Parameter xScrew;
	MaleScrew_Parameter xMaleScrew;
	FemaleScrew_Parameter xFemaleScrew;
	Shaft_Parameter xShaft;
	Sleeve_Parameter xSleeve;
	PowerFrequency_Parameter xPowerFrequency;
} Feature;
#endif

typedef struct
{
	Parameter xHighFreqPeakToPeak; //高频峰峰值
	Parameter xInputEnvEnergy;	   //输入轴包络能量
	Parameter xOutputEnvEnergy;	   //输出轴包络能量
} Base_ExtendFeature;

typedef struct
{
	Parameter xEnvEnergy;  // 轴承包络能量(外圈 1x)
	Parameter xEnv2Energy; // 轴承包络能量(外圈 2x)
	Parameter xEnv3Energy; // 轴承包络能量(外圈 3x)
} Bearing_ExtendParameter;

typedef struct
{
	Parameter x1xSideEnergy; //齿轮1x边带频能量
	Parameter x2xSideEnergy; //齿轮1x边带频能量
	Parameter x3xSideEnergy; //齿轮1x边带频能量
	Parameter xEnvEnergy;	 //齿轮包络能量
} Gear_ExtendParameter;

typedef struct
{
	Base_ExtendFeature xBaseExtFea;								   //基本扩展特征
	Bearing_ExtendParameter xBearingExtFea[ACC_MAX_BEARING_COUNT]; //轴承扩展特征
	Gear_ExtendParameter xGearExtFea[ACC_MAX_GEAR_COUNT];		   //齿轮扩展特征
} ExtendFeature;

typedef struct
{
	Parameter xActualSpd; //实时转速
	Parameter temp;		  //传感器表面温度
	Feature x;			  //基本特征x
	Feature y;			  //基本特征y
	Feature z;			  //基本特征z
	ExtendFeature xExt;	  //扩展特征x
	ExtendFeature yExt;	  //扩展特征y
	ExtendFeature zExt;	  //扩展特征z
	unsigned int odr;
	unsigned int sampSum;
	unsigned int hpfEn;
	unsigned int resolution;
} Sensor_Feature;

//齿轮 轴承定义
typedef struct
{
	float fBallCount;	//轴承滚动体个数
	float fRotateSpeed; //轴承旋转速度rpm
	float fBallDim;		//轴承滚动体直径
	float fPCD;			//轴承节圆直径
	float fWorkAngle;	//轴承工作压力角
	float fBPFO;		//轴承外圈故障频率
	float fBPFI;		//轴承内圈故障频率
	float fBSF;			//轴承滚动体故障频率
	float fFTF;			//轴承保持架故障频率
} Bearing;

typedef struct
{
	float fTeethCount;	//齿轮齿数
	float fRotateSpeed; //齿轮旋转速度rpm
	float f1xGMF;		//齿轮1x啮合频点
	float f2xGMF;		//齿轮2x啮合频点
	float f3xGMF;		//齿轮3x啮合频点
} Gear;

typedef struct
{
	float fPassFrequency;
} Fan;

typedef struct
{
	float fPassFrequency;
	float fSelfVirbFreq;
} Belt;

typedef struct
{
	float fPassFrequency;
} Screw;

typedef struct
{
	float fPassFrequency;
	float fSpeed;
} MaleScrew;

typedef struct
{
	float fPassFrequency;
	float fSpeed;
} FemaleScrew;

typedef struct
{
	float fPassFrequency;
} Shaft;

typedef struct
{
	float fRotOfAxesFrequency;
} Sleeve;

typedef struct
{
	unsigned short usInputStartFreq;
	unsigned short usInputStopFreq;
	unsigned short usOutputStartFreq;
	unsigned short usOutputStopFreq;
	unsigned short usInputSpeedEnable;
	unsigned short usOutputSpeedEnable;
	unsigned short usBearingExtendFlag;
	unsigned short usGearExtendFlag;	   //齿轮扩展特征设置标志
	unsigned short usHPFCutOffFreq;		   //巴特沃斯高通截止频率
	unsigned short usPeakVueHPFCutOffFreq; // PeakVue高通截止频率
	unsigned short usPeakVueMaxFreq;	   // PeakVue分析最大频率
	unsigned short usButterworthN;		   //巴特沃斯阶数
	unsigned short usReserved[12];		   //总共24个配置
} Base_ExtendConfig;

typedef struct
{
	unsigned short usStartFreq;
	unsigned short usStopFreq;
	unsigned short usAlias;
} Bearing_ExtendConfig;

typedef struct
{
	unsigned short usStartFreq;
	unsigned short usStopFreq;
	unsigned short usSideRange;
	unsigned short usAlias;
} Gear_ExtendConfig;

/* Exported functions --------------------------------------------------------*/
#if CONFIG_RS485_RTU > 0
void Acc_Init(void);
bool Acc_StartCollecting(unsigned char ucPort, unsigned char ucAccID);
bool Acc_GetWaveIntoExtMemory(unsigned char ucPort, unsigned char ucAccID, Sensor_Wave *pxWave,
							  void (*pFuncMemoryWrite)(unsigned long, unsigned char *, unsigned long), unsigned long ulMemoryAddress);
bool Acc_GetFeature(unsigned char ucPort, unsigned char ucAccID, Sensor_Feature *pxFeature,
					unsigned short *pusBearingStatus, unsigned short *pusGearStatus, unsigned short *pusSpeedStatus, 
					unsigned short *pusFanStatus, unsigned short *pusBeltStatus, unsigned short *pusScrewStatus, unsigned short *pusShaftStatus, 
					unsigned short *pusMaleScrewStatus, unsigned short *pusFemaleScrewStatus, unsigned short *pusSleeveStatus,unsigned short * pusOdr, 
					unsigned short * pusSampSum, unsigned short * pusHpfEn, unsigned short * pusResolution);
bool Acc_GetSensorStatus(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusCode);
bool Acc_SetBearingInfo(unsigned char ucPort, unsigned char ucAccID, unsigned char ucNumber, Bearing *pxComponent);
bool Acc_SetBearingFrequency(unsigned char ucPort, unsigned char ucAccID, unsigned char ucNumber, Bearing *pxComponent);
bool Acc_SetGearInfo(unsigned char ucPort, unsigned char ucAccID, unsigned char ucNumber, Gear *pxComponent);
bool Acc_SetGearFrequency(unsigned char ucPort, unsigned char ucAccID, unsigned char ucNumber, Gear *pxComponent);
bool Acc_SetSpeed(unsigned char ucPort, unsigned char ucAccID, unsigned long ulSpeed);
bool Acc_SetPowerFrequency(unsigned char ucPort, unsigned char ucAccID, unsigned short usPowerFrequency);
bool Acc_ClearBearingStatus(unsigned char ucPort, unsigned char ucAccID);
bool Acc_ClearGearStatus(unsigned char ucPort, unsigned char ucAccID);
bool Acc_ClearSpeedStatus(unsigned char ucPort, unsigned char ucAccID);
bool Acc_SetPinFunctionToMB(unsigned char ucPort, unsigned char ucAccID);
bool Acc_GetVersion(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusVer);
bool Acc_GetActiveFlag(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusActiveFlag);
bool Acc_GetSamplingRate(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusSamplingRate);
bool Acc_GetMode(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusMode);
bool Acc_SetMode(unsigned char ucPort, unsigned char ucAccID, unsigned short usMode);
bool Acc_SetShockThreshold(unsigned char ucPort, unsigned char ucAccID, unsigned short usThr);
/* Extend feature and config */
bool Acc_GetExtendFeature(unsigned char ucPort, unsigned char ucAccID, Sensor_Feature *pxFeature, unsigned short *pusBearingExtStatus, unsigned short *pusGearExtStatus,
						  unsigned short *pusInputExtStatus, unsigned short *pusOutputExtStatus);
bool Acc_SetBaseExtendConfig(unsigned char ucPort, unsigned char ucAccID, Base_ExtendConfig *pxExtCfg);
bool Acc_SetBearingExtendConfig(unsigned char ucPort, unsigned char ucAccID, unsigned char ucNumber, Bearing_ExtendConfig *pxExtCfg);
bool Acc_SetGearExtendConfig(unsigned char ucPort, unsigned char ucAccID, unsigned char ucNumber, Gear_ExtendConfig *pxExtCfg);
bool Acc_ClearBaseExtendStauts(unsigned char ucPort, unsigned char ucAccID);
bool Acc_ClearBearingExtendStauts(unsigned char ucPort, unsigned char ucAccID);
bool Acc_ClearGearExtendStauts(unsigned char ucPort, unsigned char ucAccID);
// Get bearing and gear alias
bool Acc_GetBearingAlias(unsigned char ucPort, unsigned char ucAccID, unsigned char ucNumber, unsigned short *pusAlias);
bool Acc_GetGearAlias(unsigned char ucPort, unsigned char ucAccID, unsigned char ucNumber, unsigned short *pusAlias);
// Set bearing and gear alias
bool Acc_SetBearingAlias(unsigned char ucPort, unsigned char ucAccID, unsigned char ucNumber, unsigned short usAlias);
bool Acc_SetGearAlias(unsigned char ucPort, unsigned char ucAccID, unsigned char ucNumber, unsigned short usAlias);
// Get fan and belt
// Set fan and belt
bool Acc_SetFanConfigure(unsigned char ucPort, unsigned char ucAccID, Fan *pxFan);
bool Acc_SetBeltConfigure(unsigned char ucPort, unsigned char ucAccID, Belt *pxBelt);
bool Acc_ClearFanStatus(unsigned char ucPort, unsigned char ucAccID);
bool Acc_ClearBeltStatus(unsigned char ucPort, unsigned char ucAccID);
bool Acc_SetFanAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short usAlias);
bool Acc_SetBeltAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short usAlias);
bool Acc_GetFanAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusAlias);
bool Acc_GetBeltAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusAlias);
// Get or set screw
bool Acc_SetScrewConfigure(unsigned char ucPort, unsigned char ucAccID, Screw *pxScrew);
bool Acc_ClearScrewStatus(unsigned char ucPort, unsigned char ucAccID);
bool Acc_SetScrewAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short usAlias);
bool Acc_GetScrewAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusAlias);
// Get or set female and male screw
bool Acc_SetMaleScrewConfigure(unsigned char ucPort, unsigned char ucAccID, Screw *pxScrew);
bool Acc_ClearMaleScrewStatus(unsigned char ucPort, unsigned char ucAccID);
bool Acc_SetMaleScrewAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short usAlias);
bool Acc_GetMaleScrewAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusAlias);
bool Acc_SetFemaleScrewConfigure(unsigned char ucPort, unsigned char ucAccID, Screw *pxScrew);
bool Acc_ClearFemaleScrewStatus(unsigned char ucPort, unsigned char ucAccID);
bool Acc_SetFemaleScrewAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short usAlias);
bool Acc_GetFemaleScrewAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusAlias);
// Get or set shaft
bool Acc_SetShaftConfigure(unsigned char ucPort, unsigned char ucAccID, Shaft *pxShaft);
bool Acc_ClearShaftStatus(unsigned char ucPort, unsigned char ucAccID);
bool Acc_SetShaftAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short usAlias);
bool Acc_GetShaftAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusAlias);

// Get or set seelve
bool Acc_SetSleeveConfigure(unsigned char ucPort, unsigned char ucAccID, Sleeve *pxSleeve);
bool Acc_ClearSleeveStatus(unsigned char ucPort, unsigned char ucAccID);
bool Acc_SetSleeveAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short usAlias);
bool Acc_GetSleeveAlias(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusAlias);

// Get sensor ODR
bool Acc_GetOdr(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusOdr);
// Get sensor sample sum
bool Acc_GetSampSum(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusSampSum);
// Get sensor HPF enable
bool Acc_GetHpfEn(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusHpfEn);
// Get sensor resolution
bool Acc_GetResolution(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusResolution);
// Set sensor ODR
bool Acc_SetOdr(unsigned char ucPort, unsigned char ucAccID, unsigned short usOdr);
// Set sensor sample sum
bool Acc_SetSampSum(unsigned char ucPort, unsigned char ucAccID, unsigned short usSampSum);
// Set sensor HPF enable
bool Acc_SetHpfEn(unsigned char ucPort, unsigned char ucAccID, unsigned short usHpfEn);

// Get anomal vibration alarm
bool Acc_GetAmlVibAlarm(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusAmlVibAlarm);
// Set anomal vibration threshold
bool Acc_SetAmlVibTh(unsigned char ucPort, unsigned char ucAccID, unsigned short usAmlVibTh);
// Set anomal vibration interval
bool Acc_SetAmlVibInterval(unsigned char ucPort, unsigned char ucAccID, unsigned short usAmlVibInterval);
// Set anomal vibration keep cnt
bool Acc_SetAmlVibKeepCnt(unsigned char ucPort, unsigned char ucAccID, unsigned short usAmlVibKeepCnt);

// set calculate feature
bool Acc_SetCalFea(unsigned char ucPort, unsigned char ucAccID, unsigned short uscalFea);

// sensor PSRAM high temperature protect feature
bool Acc_GetHighTempProtect(unsigned char ucPort, unsigned char ucAccID, unsigned short *pusHighTempProtect);
// sensor PSRAM high temperature protect feature
bool Acc_SetHighTempProtect(unsigned char ucPort, unsigned char ucAccID, unsigned short usHighTempProtect);

// sn
bool Acc_GetSerialNumber(unsigned char ucPort, unsigned char ucAccID, unsigned long *pulSN);
// predicted speed
bool Acc_GetPredictedSpeed(unsigned char ucPort, unsigned char ucAccID, Sensor_Feature *pxFeature);

// let sensor recalulete use old wave
bool Acc_SetRecalculete(unsigned char ucPort, unsigned char ucAccID, unsigned long is_recalculete);
// Speed sync
bool Acc_SetMotorARRate(unsigned char ucPort, unsigned char ucAccID, unsigned long fActual);
bool Acc_GetMotorARRate(unsigned char ucPort, unsigned char ucAccID, unsigned long *fActual);
// Motor actual speed select
bool Acc_SetMotorActualSel(unsigned char ucPort, unsigned char ucAccID, unsigned short sel);
bool Acc_GetMotorActualSel(unsigned char ucPort, unsigned char ucAccID, unsigned short *sel);
#endif

#if CONFIG_BLUETOOTH > 0
	void Acc_GetSenBaseFeatureData(unsigned char *ucData, Sensor_Feature *pxSenFea);
	void Acc_GetMotorSpeedFeatureData(unsigned char *ucData, Sensor_Feature *pxSenFea);
	void Acc_GetBearingFeatureData(bool bFlag, bool bExtendFlag ,unsigned char index, unsigned char *ucData, Sensor_Feature *pxSenFea, char num);
    void Acc_GetGearFeatureData(bool bFlag, bool bExtendFlag, unsigned char index, unsigned char *ucData, Sensor_Feature *pxSenFea ,char num);
	void Acc_GetFanFeatureData(unsigned char *ucData, Sensor_Feature *pxSenFea);
	void Acc_GetBeltFeatureData(unsigned char *ucData, Sensor_Feature *pxSenFea);
	void Acc_GetScrewFeatureData(unsigned char *ucData, Sensor_Feature *pxSenFea);
	void Acc_GetShaftFeatureData(unsigned char *ucData, Sensor_Feature *pxSenFea);
	void Acc_GetFeMaleFeatureData(unsigned char *ucData, Sensor_Feature *pxSenFea);
	void Acc_GetMaleFeatureData(unsigned char *ucData, Sensor_Feature *pxSenFea);
	void Acc_GetPowerFeatureData(unsigned char *ucData, Sensor_Feature *pxSenFea);
	void Acc_GetDeviceInfoData(unsigned char *ucData, char *psSN, char *psType, char *psCVer, char *psLogo, char *psVer);
	void Acc_GetSensorData(unsigned char *ucData, unsigned char flag, unsigned short *pusSampleRate, unsigned int *puiResolution);
	void Acc_ClearSenSpeedFeature(Sensor_Feature *pxSenFea);
	void Acc_ClearSenOutputSpeedFeature(Sensor_Feature *pxSenFea);
	void Acc_ClearSenInputSpeedFeature(Sensor_Feature *pxSenFea);
#endif
#endif
