/* Includes ------------------------------------------------------------------*/
#include "led.h"
#include "driver/gpio.h"
#include "esp_rom_gpio.h"

/* Private macros ------------------------------------------------------------*/
#define LED_COM_PIN     4
#define LED_NET_PIN     5
#define LED_PWR_PIN     6
/**
  * @brief  LED GPIO init
  * @param  None
  * @retval None
  */
void LED_Init()
{
	    // SIM 和 EC20电压使能
    esp_rom_gpio_pad_select_gpio(LED_COM_PIN);              // 选择个GPIO
	  gpio_set_direction(LED_COM_PIN, GPIO_MODE_OUTPUT);      // 把这个GPIO作为输出     
    esp_rom_gpio_pad_select_gpio(LED_NET_PIN);                         // 选择个GPIO
  	gpio_set_direction(LED_NET_PIN, GPIO_MODE_OUTPUT);                 // 把这个GPIO作为输出     
    esp_rom_gpio_pad_select_gpio(LED_PWR_PIN);                         // 选择个GPIO
  	gpio_set_direction(LED_PWR_PIN, GPIO_MODE_OUTPUT);                 // 把这个GPIO作为输出   
    LED_SetPWRStatus(true);  
    LED_SetCOMStatus(false);  
    LED_SetSTAStatus(false);
}

/**
  * @brief  LED set sta status
  * @param  bStatus
  * @retval None
  */
void LED_SetSTAStatus(bool bStatus)
{
  if(bStatus == true)
    gpio_set_level(LED_NET_PIN, 1);
  else
    gpio_set_level(LED_NET_PIN, 0);
}

/**
  * @brief  LED sta toggle
  * @param  None
  * @retval None
  */
void LED_STAToggle()
{
  static bool bStatus = false;
	if (bStatus == false)
  {
    bStatus = true;
    gpio_set_level(LED_NET_PIN, 0);
  }	
	else
  {
    bStatus = false;
		gpio_set_level(LED_NET_PIN, 1);
  }
}

/**
  * @brief  LED set com status
  * @param  bStatus
  * @retval None
  */
void LED_SetCOMStatus(bool bStatus)
{
  if(bStatus == true)
    gpio_set_level(LED_COM_PIN, 1);
  else
    gpio_set_level(LED_COM_PIN, 0); 
}

/**
  * @brief  LED com toggle
  * @param  None
  * @retval None
  */
void LED_COMToggle(void)
{
  static bool bStatus = false;
	if (bStatus == false)
  {
    bStatus = true;
    gpio_set_level(LED_COM_PIN, 0);
  }	
	else
  {
    bStatus = false;
		gpio_set_level(LED_COM_PIN, 1);
  }
}

/**
  * @brief  LED set pwr status
  * @param  None
  * @retval None
  */
void LED_SetPWRStatus(bool bStatus)
{
  if(bStatus == true)
    gpio_set_level(LED_PWR_PIN, 1);
  else 
    gpio_set_level(LED_PWR_PIN, 0);  
}


