#include "ram.h"
#include <string.h>
#include <stdint.h>
#include "acc.h"
#include "esp_log.h"

#define   WAVE_STORE_MAX_NUM       24
#define   FEATURE_STORE_MAX_NUM    7
dev_wave       *gDevWave[WAVE_STORE_MAX_NUM]         = {NULL};
dev_feature    *gDevFeature[FEATURE_STORE_MAX_NUM]   = {NULL};

unsigned char  *gMqttWaveData    = NULL;
unsigned char  *gMqttFeatureData = NULL;

void ram_init(void)
{
    for(int i = 0 ; i < WAVE_STORE_MAX_NUM ; i++ ) {
        gDevWave[i] = (dev_wave*) malloc(sizeof(dev_wave));
        if(gDevWave[i] == NULL) {
            ESP_LOGD("ram","(gDevWave malloc failed");
        } 
    }
    for(int i = 0 ; i < FEATURE_STORE_MAX_NUM; i++ ) {
        gDevFeature[i] = (dev_feature*) malloc(sizeof(dev_feature));    
        if(gDevFeature[i] == NULL) {
            ESP_LOGD("ram","gDevFeature malloc failed");
        } 
    }
    gMqttWaveData    = (unsigned char*) malloc(320000); 
    gMqttFeatureData = (unsigned char*) malloc(110000); 
    if(gMqttWaveData == NULL) {
            ESP_LOGD("ram","gMqttWaveData malloc failed");
    } 
    if(gMqttFeatureData == NULL) {
            ESP_LOGD("ram","gMqttFeatureData malloc failed");
    } 
}

dev_wave *get_dev_wave(unsigned int index)
{
    return gDevWave[index];
}

dev_feature *get_dev_fea(unsigned int index)
{
    return gDevFeature[index];
}

void write_ram_wave(unsigned int uiSensorID, unsigned long index, unsigned char *pucWriteBuffer, unsigned long ulSize)
{
    if(pucWriteBuffer == NULL || index >= WAVE_MAX_SIZE)
		return;
	memcpy(gDevWave[uiSensorID]->cWave + index, pucWriteBuffer, ulSize); 
}

void read_ram_wave(unsigned int uiSensorID, unsigned long index, unsigned char *pucReadBuffer, unsigned long ulSize)
{
    if(pucReadBuffer == NULL || index >= WAVE_MAX_SIZE)
		return;
	memcpy(pucReadBuffer, gDevWave[uiSensorID]->cWave + index, ulSize);
}

void write_ram_feature(unsigned int uiSensorID, unsigned long index, unsigned char *pucWriteBuffer, unsigned long ulSize)
{
    if(pucWriteBuffer == NULL || index >= FEATURE_MAX_SIZE)
		return;
	memcpy(gDevFeature[uiSensorID]->cFeature + index, pucWriteBuffer, ulSize); 
}

void read_ram_feature(unsigned int uiSensorID, unsigned long index, unsigned char *pucReadBuffer, unsigned long ulSize)
{
    if(pucReadBuffer == NULL || index >= FEATURE_MAX_SIZE)
		return;
	memcpy(pucReadBuffer, gDevFeature[uiSensorID]->cFeature + index, ulSize);
}

void write_mqtt_wave_data(unsigned long index, unsigned char *pucWriteBuffer, unsigned long ulSize)
{
    memcpy(gMqttWaveData + index, pucWriteBuffer, ulSize);
}

void read_mqtt_wave_data(unsigned long index, unsigned char *pucReadBuffer, unsigned long ulSize)
{
    memcpy(pucReadBuffer, gMqttWaveData + index, ulSize);
}

void write_mqtt_feature_data(unsigned long index, unsigned char *pucWriteBuffer, unsigned long ulSize)
{
    memcpy(gMqttFeatureData + index, pucWriteBuffer, ulSize);
}

void read_mqtt_feature_data(unsigned long index, unsigned char *pucReadBuffer, unsigned long ulSize) 
{
    memcpy(pucReadBuffer, gMqttFeatureData + index, ulSize);
}