#include "ts_iot01b.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/queue.h"
#include <string.h>

// 不使用此校验函数
static unsigned char CRC_Sum(unsigned char *ucData, unsigned short usSize)
{
    unsigned char sum = 0; 
    if(ucData == NULL) return 0;
    // printf("CRC:\n");
    for(int i = 0; i < usSize; i++)
    {
        printf("0x%02x", ucData[i]);
        sum += ucData[i];
    }

    // printf("\ncal sum = 0x%02x\n", sum);
    return sum;
}

// 使用此校验函数
static unsigned char CRC_Sum_sIOT01B_Format(sIOT01B_Format *ucData, unsigned short usSize)
{
    unsigned char sum = 0; 
    if(ucData == NULL) return 0;
    // printf("CRC:\n");
    sum +=ucData->ucHeader;
    sum +=(ucData->ulDevID>>8)&0x00FF;
    sum +=(ucData->ulDevID)&0x00FF;
        // printf("ulDevID=0x%04x\n", ucData->ulDevID);
    sum +=(ucData->ulDevSN>>8)&0x00FF;
    sum +=(ucData->ulDevSN)&0x00FF;
        // printf("ulDevSN=0x%04x\n", ucData->ulDevSN);
    sum +=ucData->ucStarter;
    sum +=ucData->ucCmd;
    sum +=ucData->uclen;
    for(int i=0; i < ucData->uclen; i++)
    {
        // printf("data[%d]=0x%02x\n", i, ucData->data[i]);
        sum+=ucData->data[i];
    }

    // printf("\ncal sum = 0x%02x\n", sum);
    return sum;
}

// hex dump
void message_hex(unsigned char *buf, unsigned short len)
{
    for(unsigned short i = 0; i<len; i++)
    {
        printf("0x%02x ", buf[i]);
    }
    printf("\n");
}


// Toggle the byte order of a 16-bit value
unsigned short Convert(unsigned short s) 
{
	char right, left;
	right = s& 0XFF;//低八位
	left = s >> 8;//高八位  右移8位
	s = right * 256 + left;
	return s;
}


bool Response_Package( unsigned short id, unsigned short sn, unsigned char cmd, unsigned char data_size, unsigned char *data)
{
    sIOT01B_Format package;
    bool ret;

    package.ucHeader = 0X68;        //1
    package.ulDevID = Convert(id);  //2
    package.ulDevSN = Convert(sn);  //2
    package.ucStarter = 0x68;       //1
    package.ucCmd = cmd;            //1
    package.uclen = data_size;      //1
    package.data = data;
    package.ucChecksum = CRC_Sum_sIOT01B_Format(&package, 8 + data_size);  //1
    package.ucTail = 0x16;          //1

    unsigned char *send_buf = (unsigned char*) malloc(sizeof(unsigned char) * (10 + data_size));
    
    memcpy(send_buf, &package, 8);
    memcpy(send_buf+8, package.data, data_size);
    memcpy(send_buf+8+data_size, &package.ucChecksum, 1);
    memcpy(send_buf+9+data_size, &package.ucTail, 1);

    ret = Message_Send((uint8_t*)send_buf, 10 + data_size);
    free(send_buf);
    return ret;
}

bool ACK_0AH(unsigned short id, unsigned short sn)
{
    unsigned char cmd = 0x0A;
    unsigned char data_size = 0x01;
    unsigned char data[1] = {0x01};
    return Response_Package(id, sn, cmd, data_size, data);
}

//68 00 3F 00 01 68 0C 01 02 1F 16
bool InquireStatus(unsigned short id, unsigned short sn)
{
    unsigned char cmd = 0x0C;
    unsigned char data_size = 0x01;
    unsigned char data[1] = {0x02};
    return Response_Package(id, sn, cmd, data_size, data);
}

bool Start_Detect(unsigned short id, unsigned short sn)
{
    unsigned char cmd = 0x04;
    unsigned char data_size = 0x01;
    unsigned char data[1] = {0x00};
    // printf("Send Start_Detect\n");
    return Response_Package(id, sn, cmd, data_size, data);
}

void Get_SensorIOT_Data(unsigned short id, unsigned short sn)
{
    Start_Detect(id, sn);
}

bool Stop_Detect(unsigned short id, unsigned short sn)
{
    unsigned char cmd = 0x06;
    unsigned char data_size = 0x01;
    unsigned char data[1] = {0x00};
    // printf("Send Stop_Detect\n");
    return Response_Package(id, sn, cmd, data_size, data);
}

bool Set_Threshold(unsigned short id, unsigned short sn, unsigned short threshold)
{
    unsigned char cmd = 0x02;
    unsigned char data_size = 0x02;
    union UDATA{
        unsigned short value;
        unsigned char data[2];
    }udata;

    if(threshold < 0 || threshold > 1000)
    {
        return false;
    }

    udata.value = threshold;
    printf("udata.value=%d\n", udata.value);
    udata.value = Convert(udata.value);
    printf("udata.data[0]=%d\n", udata.data[0]);
    printf("udata.data[1]=%d\n", udata.data[1]);

    return Response_Package(id, sn, cmd, data_size, udata.data);
}


bool data_frame_parsing(unsigned char *buf, int len, sIOT01B_Format* data)
{
    unsigned short count = 0;
    if(buf == NULL)
    {
        printf("buf is null");
        return false;
    }
    memcpy(&data->ucHeader, (unsigned char*)buf, sizeof(data->ucHeader));
    count += sizeof(data->ucHeader);

    memcpy(&data->ulDevID, buf + count, sizeof(data->ulDevID));
    count += sizeof(data->ulDevID);

    memcpy(&data->ulDevSN, buf + count, sizeof(data->ulDevSN));
    count += sizeof(data->ulDevSN);

    memcpy(&data->ucStarter, buf + count, sizeof(data->ucStarter));
    count += sizeof(data->ucStarter);

    memcpy(&data->ucCmd, buf + count, sizeof(data->ucCmd));
    count += sizeof(data->ucCmd);

    memcpy(&data->uclen, buf + count, sizeof(data->uclen));
    count += sizeof(data->uclen);
    // printf("count:%d\n",count);

    data->data = buf + count;
    count += data->uclen;
    // printf("%x %x %x %x\n",data->data[0],data->data[1],data->data[2],data->data[3]);

    memcpy(&data->ucChecksum, buf + count, sizeof(data->ucChecksum));
    count += sizeof(data->ucChecksum);

    memcpy(&data->ucTail, buf + count, sizeof(data->ucTail));
    count += sizeof(data->ucTail);

    data->ulDevID = Convert(data->ulDevID);  
    data->ulDevSN = Convert(data->ulDevSN);  

    // basic check
    if(data->ucHeader != 0x68 || data->ucTail != 0x16)
    {
        // printf("Header[%2x] & Tail[%2x] Error.\n", data->ucHeader, data->ucTail);
        return false;
    }

    // crc sum check
    if(CRC_Sum_sIOT01B_Format(data, 8 + data->uclen) != data->ucChecksum)
    {
        printf("CRC_Sum_sIOT01B_Format check fail.\n");
        return false;
    }

    return true;
}


eIOT01B_Info IOT01B_Data_Parsing(void *ulData, unsigned short usSize)
{
    return EIOT01B_OK;
}


eIOT01B_Info IOT01B_Get_Collect_Data(sIOT01B_Format *df, unsigned short *value)
{
    unsigned int sum = 0;
    // printf("df.data len = %d\n", df->uclen);
    
    for(int i = 0; i < df->uclen; i++) // 合约中规定的数据是固定两个字节，将两字节转为十进制之后除以10，得到百分比
    {
        // printf("df->data[%d]=0x%x\n", i, df->data[i]);
        sum +=  df->data[i] << (8 * (df->uclen -1 - i));
        // printf("%d sum = %d\n",i, sum);
    }
    // printf("Get Sum:[%d]\n", sum);
    *value = sum;
    return EIOT01B_OK;
}


void queueInitFifo(unsigned short *fifo_tail, unsigned short fifo_data[], bool *isFull)
{
    (*fifo_tail) = 0;
    for(unsigned short i = 0; i < FIFO_SIZE; i++)
    {
        fifo_data[i] = 0;
    }
    (*isFull) = false;
}

unsigned short calculateAverage(unsigned short fifo_data[], bool isFull) 
{
    if(isFull != true) return 0;
    unsigned short sum = 0;

    for (int i = 0; i < FIFO_SIZE; i++) 
    {
        sum += fifo_data[i];
    }
    printf("sum: %d\n", sum);
    return sum / FIFO_SIZE;
}

void enqueueDataFifo(unsigned short *fifo_tail, unsigned short *fifo_data, int data, bool *isFull)
{
    fifo_data[(*fifo_tail)] = data;
    (*fifo_tail) = (*fifo_tail) + 1;
    if ((*fifo_tail) >= FIFO_SIZE)
    {
        (*fifo_tail) = 0;
        *isFull = true;
        // printf("\n");
        // for(int i = 0; i< FIFO_SIZE; i++)
        // {
        //     printf("%d ", fifo_data[i]);
        // }
        // printf("\n");
    }
}


unsigned short maxDataWireRope(unsigned short fifo_data[], bool isFull) 
{
    if(isFull != true) return 0;

    int max = 0;
    for (int i = 0; i < FIFO_SIZE; i++) 
    {
        if (fifo_data[i] > max) 
        {
            max = fifo_data[i];
        }
    }
    return max;
}

int getAlarm( int alarmRecord, bool isFull)
{
    if(isFull != true) return 0;

    return (alarmRecord);
}

void setAlarm( unsigned long *alarmRecord)
{
    if((*alarmRecord) >= 999999)
    {
        (*alarmRecord)  = 999999;
    }
    else
    {
        (*alarmRecord)++;
    }
}






