#ifndef __TS_IOT01B__H__
#define __TS_IOT01B__H__

#include <stdint.h>
#include <stdbool.h>

// Slave Command
#define     WIREROPE_CMD_WAIT_AWAKE             0x01
#define     WIREROPE_CMD_ACK_OR_NACK            0x03
#define     WIREROPE_CMD_HEARTBEAT              0x05
#define     WIREROPE_CMD_DATA_PERCENT           0x07
#define     WIREROPE_CMD_SYSTEM_PARAM           0x09
#define     WIREROPE_CMD_WORK_STATUS_MESSAGE    0x0B
#define     WIREROPE_CMD_WORK_PARAM             0x0D

// Master Command
#define     MASTER_ACK                          0x0A
#define     MASTER_START_DETECT                 0x04


// Frame Define 
#define FRAME_HEADER    0x68
#define FRAME_START     0x68
#define FRAME_TAIL      0x16


#define TCP_MESSAGE_MAX_LEN     280

#define EACH_GROUP_MAX_NUM       4
#define WIREROPE_GROUP_MAX_NUM   2



#define FIFO_SIZE 100

typedef enum {
    EIOT01B_OK = 0,
    EIOT01B_TIMEOUT,
    EIOT01B_NO_COMPLETE
}eIOT01B_Info;

typedef enum{
    SLAVER_IDEL_STATUS = 0,
    SLAVER_POWER_ON_STATUS, 
    SLAVER_ACK_STATUS,
    SLAVER_HEARTBEAT_STATUS,
    SLAVER_WIREROPE_DETECT_STATUS
}SLAVER_STATUS;

typedef struct {
    unsigned char ucHeader;
    unsigned short ulDevID;
    unsigned short ulDevSN;
    unsigned char ucStarter;
    unsigned char ucCmd;
    unsigned char uclen;
    unsigned char *data;
    unsigned char ucChecksum;
    unsigned char ucTail;
}__attribute__((packed)) sIOT01B_Format;


typedef struct {
    unsigned char rec_len;
    unsigned char message[TCP_MESSAGE_MAX_LEN];
}Tcp_Message;


eIOT01B_Info IOT01B_Set_Upload_Threshold(unsigned long ulDevId);
eIOT01B_Info IOT01B_Set_Start_Collect(unsigned long ulDevId);
eIOT01B_Info IOT01B_Set_Stop_Collect(unsigned long ulDevId);
eIOT01B_Info IOT01B_Get_Collect_Data(sIOT01B_Format *df, unsigned short *value);

eIOT01B_Info IOT01B_Data_Parsing(void *ulData, unsigned short usSize);
void message_deal_start_task(void);
void message_hex(unsigned char *buf, unsigned short len);
bool Message_Send(uint8_t *smessage, int len);
bool data_frame_parsing(unsigned char *buf, int len, sIOT01B_Format* data);
eIOT01B_Info IOT01B_Data_Parsing(void *ulData, unsigned short usSize);
bool ACK_0AH(unsigned short id, unsigned short sn);
bool InquireStatus(unsigned short id, unsigned short sn);
bool Set_Threshold(unsigned short id, unsigned short sn, unsigned short threshold);
bool Start_Detect(unsigned short id, unsigned short sn);
bool Stop_Detect(unsigned short id, unsigned short sn);
void Get_SensorIOT_Data(unsigned short id, unsigned short sn);
void IOT_SetConfig();
// unsigned long getIotConnectNum(void);

void queueInitFifo(unsigned short *fifo_tail, unsigned short fifo_data[], bool *isFull);
unsigned short calculateAverage(unsigned short fifo_data[], bool isFull) ;
unsigned short maxDataWireRope(unsigned short fifo_data[], bool isFull) ;
int getAlarm( int alarmRecord, bool isFull);
void setAlarm( unsigned long *alarmRecord);
void enqueueDataFifo(unsigned short *fifo_tail, unsigned short *fifo_data, int data, bool *isFull) ;



#endif
