/*
 * FreeModbus Libary: BARE Port
 * Copyright (C) 2006 Christian Walter <wolti@sil.at>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * File: $Id: portserial.c,v 1.1 2006/08/22 21:35:13 wolti Exp $
 */

#include "port.h"

/* ----------------------- Modbus includes ----------------------------------*/
#include "mb.h"
#include "mbport.h"

/* ----------------------- User includes ------------------------------------*/
#include <string.h>

#include "wit_config.h"
#include "usart.h"

/* ----------------------- Private macros -----------------------------------*/
#define MB_UART_PRIORITY		0

/* ----------------------- Private variables --------------------------------*/


/* ----------------------- static functions ---------------------------------*/
static void prvvUARTTxReadyISR(  UCHAR ucModbusPort );
static void prvvUARTR1ISR( void );
static void prvvUARTR2ISR( void );

static void prvvRS485ControlPinInit(UCHAR ucModbusPort);
static void prvvRS485SetDirection(BOOL bTx, UCHAR ucModbusPort);
static void prvvTxeHandler_Uart1(void);
static void prvvTcHandler_Uart1(void);
static void prvvTxeHandler_Uart2(void);
static void prvvTcHandler_Uart2(void);

#define UARTRXISR(X)			prvvUARTR ## X ## ISR
#define TXEHANDLER_UARTX(X)     prvvTxeHandler_Uart ## X
#define TCHANDLER_UARTX(X)      prvvTcHandler_Uart ## X

#define GPIO_RS485_EN0       GPIO_NUM_45
#define GPIO_RS485_EN1       GPIO_NUM_8

/* ----------------------- Start implementation -----------------------------*/
//配置对应串口所占BUFF数组的缓存索引
inline unsigned char MB_PORT_TO_INDEX(unsigned char ucUartPort)
{
    switch(ucUartPort)
	{
		case 1:
			return 0;
		case 2:
			return 1;
		default:
			return 0;
    }

}
	
void
vMBPortSerialEnable( BOOL xRxEnable, BOOL xTxEnable,UCHAR ucModbusPort)
{
    /* If xRXEnable enable serial receive interrupts. If xTxENable enable
     * transmitter empty interrupts.
     */
	if (xRxEnable == TRUE)
		prvvRS485SetDirection(FALSE,ucModbusPort);
	else if (xTxEnable == TRUE)
		prvvRS485SetDirection(TRUE,ucModbusPort);
	else if (xTxEnable == FALSE && xRxEnable == FALSE)
		prvvRS485SetDirection(TRUE,ucModbusPort);
	
	UARTx_EnableIT(ucModbusPort, false, (bool)xTxEnable, (bool)xTxEnable, (bool)xRxEnable);
}

BOOL
xMBPortSerialInit( UCHAR ucPORT, ULONG ulBaudRate, UCHAR ucDataBits, eMBParity eParity )
{
	prvvRS485ControlPinInit(ucPORT);
	UARTx_Init(ucPORT, ulBaudRate, eParity, eParity > 0 ? UART_STOP_BITS_1 : UART_STOP_BITS_2);
    switch(ucPORT)
    {
        case 1:
             UARTx_SetCallBack(ucPORT, NULL, prvvTxeHandler_Uart1, prvvTcHandler_Uart1, prvvUARTR1ISR);
             UARTx_SetPriority(ucPORT, MB_UART_PRIORITY);
             break;
        case 2:
             UARTx_SetCallBack(ucPORT, NULL, prvvTxeHandler_Uart2, prvvTcHandler_Uart2, prvvUARTR2ISR);
             UARTx_SetPriority(ucPORT, MB_UART_PRIORITY );
             break;
        default:
            break;
    }
    return TRUE;
}

BOOL
xMBPortSerialPutByte( CHAR ucByte, UCHAR ucModbusPort)
{
    /* Put a byte in the UARTs transmit buffer. This function is called
     * by the protocol stack if pxMBFrameCBTransmitterEmpty( ) has been
     * called. */
	UARTx_SendToDR(ucModbusPort, ucByte);
    return TRUE;
}

BOOL
xMBPortSerialGetByte( CHAR * pucByte,UCHAR ucModbusPort )
{
    /* Return the byte in the UARTs receive buffer. This function is called
     * by the protocol stack after pxMBFrameCBByteReceived( ) has been called.
     */
	*pucByte = UARTx_Receive(ucModbusPort);
    return TRUE;
}

/* Create an interrupt handler for the transmit buffer empty interrupt
 * (or an equivalent) for your target processor. This function should then
 * call pxMBFrameCBTransmitterEmpty( ) which tells the protocol stack that
 * a new character can be sent. The protocol stack will then call 
 * xMBPortSerialPutByte( ) to send the character.
 */
static void prvvUARTTxReadyISR( UCHAR ucModbusPort )
{
    pxMBFrameCBTransmitterEmpty(ucModbusPort );
}

/* Create an interrupt handler for the receive interrupt for your target
 * processor. This function should then call pxMBFrameCBByteReceived( ). The
 * protocol stack will then call xMBPortSerialGetByte( ) to retrieve the
 * character.
 */
static void prvvUARTR1ISR(  )
{
    pxMBFrameCBByteReceived(1);
}
static void prvvUARTR2ISR(  )
{
    pxMBFrameCBByteReceived(2);
}
/* ----------------------- User functions -----------------------------------*/
static void prvvRS485ControlPinInit( UCHAR ucModbusPort)
{
	switch(ucModbusPort)
	{
		case 1:
			esp_rom_gpio_pad_select_gpio(GPIO_RS485_EN0);  
			gpio_set_direction(GPIO_RS485_EN0, GPIO_MODE_OUTPUT); 
			break;
		case 2:
			esp_rom_gpio_pad_select_gpio(GPIO_RS485_EN1);   
	        gpio_set_direction(GPIO_RS485_EN1, GPIO_MODE_OUTPUT);
		    break;
		default:
			break;
	}
}

static void prvvRS485SetDirection(BOOL bTx,UCHAR ucModbusPort)
{
	switch(ucModbusPort)
	{
		case 1:
			bTx == TRUE ? gpio_set_level(GPIO_RS485_EN0, 1) : gpio_set_level(GPIO_RS485_EN0, 0);
			break;
		case 2 :
			bTx == TRUE ? gpio_set_level(GPIO_RS485_EN1, 1) : gpio_set_level(GPIO_RS485_EN1, 0);
			break;
		default:
			break;
	}
	printf("bTx %d, port %d \r\n", bTx, ucModbusPort);
}

static void prvvTxeHandler_Uart1()
{
	prvvUARTTxReadyISR(1);
	UARTx_EnableIT(1, false, false, true, false);
}

static void prvvTcHandler_Uart1()
{
	prvvUARTTxReadyISR(1);
}

static void prvvTxeHandler_Uart2()
{
	prvvUARTTxReadyISR(2);
	UARTx_EnableIT(2, false, false, true, false);
}

static void prvvTcHandler_Uart2()
{
	prvvUARTTxReadyISR(2);
}
