/**
******************************************************************************
* @file         tcp_server.c                                         
* @version      V1.0                          
* @date         2018-06-18                 
* @brief        tcp服务器数据回环测试        
*
* @company      深圳炜世科技有限公司
* @information  WIZnet W5100S官方代理商，全程技术支持，价格优势大！
* @website      www.wisioe.com       
* @forum        www.w5100s.com
* @qqGroup      579842114                                                     
******************************************************************************
*/
#include <stdio.h>
#include <string.h>
#include <stdbool.h>
#include "socket.h"
#include "wizchip_conf.h"
#include "w5100s_conf.h"
#if CONFIG_W5100S_SPI_ETHERNET > 0
    #include "eth.h"
#endif
#include "ts_iot01b.h"

#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/queue.h"
#include "freertos/timers.h"

#include "tcp_server.h"
#include "wit.h"

uint16 local_port = 11111;   // 定义本地端口11111

uint8_t g_IOTBuffer[200];
Tcp_Message g_Tcp_Message;
extern QueueHandle_t xMessageQueueHandle;
TimerHandle_t xSensorTimeoutTimer = NULL;
TimerHandle_t xSensorDataUpdateTimer = NULL;
TCP_STATUS g_TcpServerStatus = TCP_SERVER_SOCKET_CLOSE;
extern TimerHandle_t g_xDataSendTimer[];


void do_tcp_server(void)
{
    uint8_t ret;
    uint16 len;
    portBASE_TYPE ret_pB;
    // my_socket(2, Sn_MR_TCP, 11111, 0);

    while(1)
    {
        len = 0;
        ret = getSn_SR(SOCK_TCPS);

        switch(ret)                             // 获取socket的状态
        {
            case SOCK_CLOSED:                                     // socket处于关闭状态
           {
                my_socket(SOCK_TCPS ,Sn_MR_TCP, PORTS, 0);   // 打开socket
                g_TcpServerStatus = TCP_SERVER_SOCKET_CREATE;
                break;
           }
            
            case SOCK_INIT:                                       // socket已初始化状态
            {   
                listen(SOCK_TCPS);                                  // socket建立监听
                g_TcpServerStatus = TCP_SERVER_INIT;
                break;
            }
            
            case SOCK_ESTABLISHED:                               // socket处于连接建立状态
            {
                if(getSn_IR(SOCK_TCPS) & Sn_IR_CON)
                {
                    setSn_IR(SOCK_TCPS, Sn_IR_CON);                   // 清除接收中断标志位
                }

                len = getSn_RX_RSR(SOCK_TCPS);                      // 定义len为已接收数据的长度    
                if(len > 0)
                {
                    if(len > 280)
                    {
                        len = 265;
                    }
                    recv(SOCK_TCPS, g_Tcp_Message.message, len);                            // 接收来自Client的数据
                    g_Tcp_Message.rec_len = len;                                 // 添加字符串结束符
                    // printf("sock receive len[%d]: %s\n", g_Tcp_Message.rec_len, g_Tcp_Message.message);
                    if (xMessageQueueHandle != NULL)
                    {

                        ret_pB =  xQueueSend( xMessageQueueHandle, &g_Tcp_Message, 0); // 将数据发送到消息队列中
                        if(ret_pB == pdTRUE)
                        {
                            // printf("xQSend OK\n");
                        }
                        else
                            printf("xQSend Exx\n");

                    }
                    else
                    {
                        printf("xMessageQueueHandle is null.\n");
                    }
                    // send(SOCK_TCPS, g_Tcp_Message.message, len);// 向Client发送数据
                    // message_hex(g_Tcp_Message.message, len);
                    memset(&g_Tcp_Message, 0, sizeof(g_Tcp_Message));
                }
                g_TcpServerStatus = TCP_SERVER_EST;
                break;
            }
            
            case SOCK_CLOSE_WAIT:                                 // socket处于等待关闭状态
            {
                disconnect(SOCK_TCPS);                              // 断开当前TCP连接
                close(SOCK_TCPS);                                   // 关闭当前所使用socket
                g_TcpServerStatus = TCP_SERVER_SOCKET_CLOSE;
                break;
            }
            
        }
        vTaskDelay(10);
    }
}

bool Message_Send(uint8_t *smessage, int len)
{
    if(g_TcpServerStatus == TCP_SERVER_EST)
    {
        send(SOCK_TCPS, smessage, len);                         // 向Client发送数据
        return true;
    }
    else
    {
        // printf("No Client ESTABLISHED [%x]\n", g_TcpServerStatus);
        return false;
    }
}


void tcp_server_task_start(void)
{
    printf("1 start\n");
    xTaskCreate(do_tcp_server, "tcp_server_task", 2048, NULL, 5, NULL);
}

void message_deal_start_task(void)
{
    printf("2 start\n");
    
    while(xMessageQueueHandle == NULL)
    {
        xMessageQueueHandle = xQueueCreate(100, sizeof(Tcp_Message));
    }
    xTaskCreate(Sensor_Status_Machine, "StatusMachine", 5120, NULL, 5, NULL);
    xTaskCreate(Message_Deal_Function, "MessageDealFunction", 10240, NULL, 5, NULL);
}
