/* Includes ------------------------------------------------------------------*/
#include <string.h>
#include <stdlib.h>
#include "celluar_event.h"

/* Private variables --------------------------------------------------------*/
static Celluar_EventQueue g_xCelluarEventQueue;

/* Private functions --------------------------------------------------------*/
static bool Celluar_EventQueueIsEmpty(void);
static bool Celluar_EventQueueIsFull(void);

/**
  * @brief  Celluar post event
  * @param  eEvent: event type
  * @param  pucData: data pointer can be null
  * @param  ulLength: length of data
  * @retval None
  */
void Celluar_EventPost(Celluar_EventType eEvent, unsigned char *pucData, unsigned long ulLength)
{
	if (!Celluar_EventQueueIsFull())
	{
		g_xCelluarEventQueue.eEventArray[g_xCelluarEventQueue.ulRear] = eEvent;
		if (pucData != NULL)
		{
			Celluar_EventMessage *pxMessage;
			
			//malloc
			pxMessage = &g_xCelluarEventQueue.xMessage[g_xCelluarEventQueue.ulRear];
			pxMessage->pucPayload 	= (unsigned char *)malloc(ulLength + 1);
			if (pxMessage->pucPayload != NULL)
			{
				pxMessage->ulLength		= ulLength;
				memcpy(pxMessage->pucPayload, pucData, pxMessage->ulLength);
				pxMessage->pucPayload[ulLength] = '\0';
			}
		}
		g_xCelluarEventQueue.ulRear = (g_xCelluarEventQueue.ulRear + 1) % CELLUAR_MAX_EVENT_NUM;
	}
}

/**
  * @brief  Celluar get event
  * @param  peEvent: current event if has
  * @param  pucData: return the data about event
  * @param  pulLength: length of daata
  * @retval true: event happend false: no event
  */
bool Celluar_EventGet(Celluar_EventType *peEvent, unsigned char **ppucData, unsigned long *pulLength)
{
	if (!Celluar_EventQueueIsEmpty())
	{
		Celluar_EventMessage *pxMessage;
			
		*peEvent 	= g_xCelluarEventQueue.eEventArray[g_xCelluarEventQueue.ulFront];
		pxMessage 	= &g_xCelluarEventQueue.xMessage[g_xCelluarEventQueue.ulFront];
		if (pxMessage->pucPayload != NULL)
		{
			if (ppucData != NULL)
				*ppucData = pxMessage->pucPayload;
			if (pulLength != NULL)
				*pulLength = pxMessage->ulLength;
		}
		else
		{
			*pulLength = 0;
		}
		
		return true;
	}
	return false;
}

/**
  * @brief  celluar delete current event
  * @retval true: delete success / false: no data ro delete
  */
bool Celluar_EventDel()
{
	if (!Celluar_EventQueueIsEmpty())
	{
		Celluar_EventMessage *pxMessage;
		
		pxMessage 	= &g_xCelluarEventQueue.xMessage[g_xCelluarEventQueue.ulFront];
		if (pxMessage->pucPayload != NULL)
		{
			//Free
			free(pxMessage->pucPayload);
			pxMessage->pucPayload 	= NULL; 
			pxMessage->ulLength 	= 0;
		}
		
		g_xCelluarEventQueue.ulFront = (g_xCelluarEventQueue.ulFront + 1) % CELLUAR_MAX_EVENT_NUM;
		return true;
	}
	return false;
}

/**
  * @brief  queue is empty?
  * @param  peEvent: current event if has
  * @retval true: empty / false: not empty
  */
static bool Celluar_EventQueueIsEmpty()
{
	return g_xCelluarEventQueue.ulFront == g_xCelluarEventQueue.ulRear ? true : false;
}

/**
  * @brief  queue is full?
  * @param  None
  * @retval true: full / false: not full
  */
static bool Celluar_EventQueueIsFull()
{
	return (g_xCelluarEventQueue.ulRear + 1) % CELLUAR_MAX_EVENT_NUM == g_xCelluarEventQueue.ulFront ? true : false;
}

