/* Includes ------------------------------------------------------------------*/
#include <string.h>
#include <stdlib.h>
#include "list_queue.h"

List_Queue List_QueueCreate(unsigned long ulQueueSize, unsigned long ulBufferSize)
{
	List_Queue xListQueue;
	int i;
	
	memset(&xListQueue, 0, sizeof(List_Queue));
	xListQueue.ulQueueSize = ulQueueSize;
	xListQueue.ulBufferSize = ulBufferSize;
	xListQueue.bQueueFull = true;
	xListQueue.xReceiveBuffer = (List_Buffer *)malloc(ulQueueSize * sizeof(List_Buffer));
	
	if (xListQueue.xReceiveBuffer != NULL)
	{
		for (i = 0;i < ulQueueSize;i++)
		{
			xListQueue.xReceiveBuffer[i].ucBuffer = (unsigned char *)malloc(ulBufferSize);
			
			if (xListQueue.xReceiveBuffer[i].ucBuffer == NULL)
			{
				break;
			}
			
			if (i == ulQueueSize - 1)
				xListQueue.bQueueFull = false;
		}
	}
	
	return xListQueue;
}

/**
  * @brief  List queue is empty
  * @param  pxQueue: Queue pointer
  * @retval true (empty) / false
  */
bool List_QueueIsEmpty(List_Queue *pxQueue)
{
	if (pxQueue != NULL)
		return (bool)(!pxQueue->bQueueFull && pxQueue->ulFront == pxQueue->ulRear);
	
	return false;
}

/**
  * @brief  List queue is full
  * @param  pxQueue: Queue pointer
  * @retval None
  */
bool List_QueueIsFull(List_Queue *pxQueue)
{
	if (pxQueue != NULL)
		return pxQueue->bQueueFull;
	
	return true;
}

/**
  * @brief  List queue push
  * @param  pxQueue: Queue pointer
  * @retval None
  */
void List_QueuePush(List_Queue *pxQueue)
{
	if (!List_QueueIsFull(pxQueue))
	{
		pxQueue->ulRear = (pxQueue->ulRear + 1) % pxQueue->ulQueueSize;
		pxQueue->bQueueFull = (pxQueue->ulFront == pxQueue->ulRear ? true : pxQueue->bQueueFull);
	}
}

/**
  * @brief  List queue pop 0
  * @param  pxQueue: Queue pointer
  * @retval None
  */
void List_QueuePop(List_Queue *pxQueue)
{
	if (!List_QueueIsEmpty(pxQueue))
	{
		memset(pxQueue->xReceiveBuffer[pxQueue->ulFront].ucBuffer, 0, pxQueue->xReceiveBuffer[pxQueue->ulFront].ulBufferLen);
		pxQueue->xReceiveBuffer[pxQueue->ulFront].ulBufferLen = 0;
		pxQueue->ulFront = (pxQueue->ulFront + 1) % pxQueue->ulQueueSize;
		pxQueue->bQueueFull = false;
	}
}




