/* Includes ------------------------------------------------------------------*/
#include "seq_queue.h"
#include <stdlib.h>

/**
  * @brief  create queue
  * @param  size
  * @retval queue
  */
SeqQueue SeqQueue_Create(unsigned long ulQueueSize)
{
	SeqQueue xQueue;
	
	xQueue.ulFront = 0;
	xQueue.ulRear = 0;
	xQueue.pucData = (unsigned char *)malloc(ulQueueSize);
	if (xQueue.pucData != NULL)
	{
		xQueue.ulQueueSize = ulQueueSize;
	}
	else
	{
		xQueue.ulQueueSize = 0;
	}
	
	return xQueue;
}

/**
  * @brief  release queue
  * @param  pxQueue
  * @retval void
  */
void SeqQueue_Release(SeqQueue *pxQueue)
{
	SeqQueue_Clear(pxQueue);
	if (pxQueue->pucData != NULL)
	{
		pxQueue->ulQueueSize = 0;
		free(pxQueue->pucData);
	}
}

/**
  * @brief  Queue is empty?
  * @param  pxQueue
  * @retval true / false
  */
bool SeqQueue_IsEmpty(SeqQueue *pxQueue)
{
	return pxQueue->ulFront == pxQueue->ulRear ? true : false;
}
	
/**
  * @brief  Queue is full?
  * @param  pxQueue
  * @retval true / false
  */
bool SeqQueue_IsFull(SeqQueue *pxQueue)
{
	return (pxQueue->ulRear + 1) % pxQueue->ulQueueSize == pxQueue->ulFront ? true : false;
}

/**
  * @brief  Enqueue
  * @param  pxQueue
  * @param  ucData
  * @retval true / false
  */
bool SeqQueue_EnQueue(SeqQueue *pxQueue, unsigned char ucData)
{
	if (!SeqQueue_IsFull(pxQueue) && pxQueue->pucData != NULL)
	{
		pxQueue->pucData[pxQueue->ulRear] = ucData;
		pxQueue->ulRear = (pxQueue->ulRear + 1) % pxQueue->ulQueueSize;
		return true;
	}
	
	return false;
}

/**
  * @brief  Dequeue
  * @param  pxQueue
  * @retval None
  */
bool SeqQueue_DeQueue(SeqQueue *pxQueue, unsigned char *pucData)
{
	if (!SeqQueue_IsEmpty(pxQueue) && pxQueue->pucData != NULL)
	{
		*pucData = pxQueue->pucData[pxQueue->ulFront];
		pxQueue->ulFront = (pxQueue->ulFront + 1) % pxQueue->ulQueueSize;
		return true;
	}
	
	return false;
}

/**
  * @brief  Get length of length
  * @param  pxQueue
  * @retval None
  */
unsigned long SeqQueue_GetLength(SeqQueue *pxQueue)
{
	return ((long)(pxQueue->ulRear - pxQueue->ulFront) + pxQueue->ulQueueSize) % pxQueue->ulQueueSize;
}

/**
  * @brief  Get rest length of length
  * @param  pxQueue
  * @retval None
  */
unsigned long 	SeqQueue_GetRestLength(SeqQueue *pxQueue)
{
	return pxQueue->ulQueueSize - SeqQueue_GetLength(pxQueue);
}

/**
  * @brief  Clear seq
  * @param  pxQueue
  * @retval None
  */
void SeqQueue_Clear(SeqQueue *pxQueue)
{
	pxQueue->ulFront = 0;
	pxQueue->ulRear = 0;
}


