#ifndef MQTT_PORT_H
#define MQTT_PORT_H
/* Includes ------------------------------------------------------------------*/
#include "common.h"
/* User Includes -------------------------------------------------------------*/
#include "wit_config.h"
#if CONFIG_W5100S_SPI_ETHERNET > 0
#include "eth.h"
#elif CONFIG_EC200S_USB_4G > 0
#include "celluar.h"
#include "celluar_socket.h"
#endif


//Buffer Size
#define MQTT_SEND_BUFFER_SIZE								(1 * 1024)
#define MQTT_READ_BUFFER_SIZE								(9 * 1024)

#if CONFIG_W5100S_SPI_ETHERNET > 0
#define MQTT_MAX_SOCKET_COUNT								(1)
#elif CONFIG_EC200S_USB_4G > 0
#define MQTT_MAX_SOCKET_COUNT								(CELLUAR_SOCKET_MAX_COUNT)
#endif

#if CONFIG_W5100S_SPI_ETHERNET > 0
#define MQTT_TCP_MODE_NAME									"TCPE"
#elif CONFIG_EC200S_USB_4G > 0
#define MQTT_TCP_MODE_NAME									"TCP"

#endif

typedef enum
{
	MQTT_TRUE     = 0,
	MQTT_FALSE    = 1,
	MQTT_TIMEOUT  = 2
} MQTT_Status;

/* Exported functions --------------------------------------------------------*/
long 		MQTT_CreateSocket(char *pcMode, char *pcHost, char *pcPort);
void 		MQTT_ReleaseSocket(unsigned long ulSocketID);
void 		MQTT_ReconnectFuncAssignFor(bool (*pFunction)(unsigned long));
void 		MQTT_StatusChangedFuncAssignFor(void (*pFunction)(unsigned long, bool));
void 		MQTT_ClearSeqQueue(unsigned long ulSocketID);
long 		MQTT_RecvFromSeqQueue(unsigned long ulSocketID, unsigned char *pucBuffer, unsigned long ulSize);
MQTT_Status MQTT_SendData(unsigned long ulSocketID, unsigned char *pucData, unsigned long ulLen);
MQTT_Status MQTT_GetSocketStatus(unsigned long ulSocketID);
MQTT_Status MQTT_SocketConnect(unsigned long ulSocketID);
MQTT_Status MQTT_SocketDisconnect(unsigned long ulSocketID);
void 		MQTT_SetSocketStatus(unsigned long ulSocketID, bool bStatus);
#endif



